void MarkChar();
void MarkString();
void SetChar();
void SetString();

struct font_entry *getcurrentfontent();
struct font_entry *currentfontent();

void realf_setchar();
void realf_setstring();
void virf_setchar();
void virf_setstring();
