#define	NJSUBS		33
#define	JSF_HIRA	(unsigned short)3

#define	JSTFM_MAGIC	142857

extern char *jsf_names[];
#define	jsubfontname(f)	jsf_names[f]
extern int foundjsubf;

#define	NCSUBS		47
#define	CSUB_SEP	'C'
